@@ -1,10 +1,13 @@
 from django.conf import settings
-from django.contrib.auth.models import AnonymousUser
+from django.contrib.auth.models import AnonymousUser, User
 from django.db import models
+from django.db.models.base import ModelBase
 from django.urls import get_resolver
 from django.utils.decorators import classonlymethod
 from guardian.shortcuts import get_perms
 
+User._meta.rdf_type = "foaf:user"
+
 
 class Model(models.Model):
 
@@ -36,8 +39,12 @@ class Model(models.Model):
         return cls.__clean_path(r_id)
 
     @classonlymethod
-    def slug_field(cls, instance):
-        view_name = '{}-detail'.format(instance._meta.object_name.lower())
+    def slug_field(cls, instance_or_model):
+        if isinstance(instance_or_model, ModelBase):
+            object_name = instance_or_model.__name__.lower()
+        else:
+            object_name = instance_or_model._meta.object_name.lower()
+        view_name = '{}-detail'.format(object_name)
         slug_field = '/{}'.format(get_resolver().reverse_dict[view_name][0][0][1][0])
         if slug_field.startswith('/'):
             slug_field = slug_field[1:]
@@ -113,7 +120,6 @@ class Model(models.Model):
         return [{'mode': {'@type': name.split('_')[0]}} for name in permissions]
 
 
-
 class LDPSource(models.Model):
     container = models.URLField()
     federation = models.CharField(max_length=255)
@@ -128,18 +134,3 @@ class LDPSource(models.Model):
 
     def __str__(self):
         return "{}: {}".format(self.federation, self.container)
-
-
-class LDNotification(models.Model):
-    user = models.ForeignKey(settings.AUTH_USER_MODEL)
-    author = models.URLField()
-    object = models.URLField()
-    type = models.CharField(max_length=255)
-    summary = models.TextField()
-    date = models.DateTimeField(auto_now_add=True)
-
-    class Meta:
-        permissions = (
-            ('view_todo', 'Read'),
-            ('control_todo', 'Control'),
-        )
