@@ -1,4 +1,5 @@
 from collections import OrderedDict, Mapping
+from typing import Any
 from urllib import parse
 
 from django.core.exceptions import ImproperlyConfigured
@@ -9,7 +10,7 @@ from django.utils.encoding import uri_to_iri
 from rest_framework.exceptions import ValidationError
 from rest_framework.fields import SkipField, empty
 from rest_framework.fields import get_error_detail, set_value
-from rest_framework.relations import HyperlinkedRelatedField, ManyRelatedField, MANY_RELATION_KWARGS
+from rest_framework.relations import HyperlinkedRelatedField, ManyRelatedField, MANY_RELATION_KWARGS, Hyperlink
 from rest_framework.serializers import HyperlinkedModelSerializer, ListSerializer, ModelSerializer
 from rest_framework.settings import api_settings
 from rest_framework.utils import model_meta
@@ -21,6 +22,8 @@ from djangoldp.models import Model
 
 
 class LDListMixin:
+    child_attr = 'child'
+
     def to_internal_value(self, data):
         try:
             data = data['ldp:contains']
@@ -30,7 +33,8 @@ class LDListMixin:
             data = [data]
         if isinstance(data, str) and data.startswith("http"):
             data = [{'@id': data}]
-        return [self.child.to_internal_value(item) for item in data]
+
+        return [getattr(self, self.child_attr).to_internal_value(item) for item in data]
 
     def to_representation(self, value):
         return {'@id': self.id,
@@ -56,7 +60,7 @@ class LDListMixin:
                     object_list))
             else:
                 container_id = Model.container_id(self.parent.instance)
-                obj = next(filter(lambda o: container_id in o[self.parent.url_field_name], object_list))
+                obj = next(filter(lambda o: container_id.lstrip('/') in o[self.parent.url_field_name], object_list))
             list = super().get_value(obj)
             try:
                 list = next(
@@ -119,7 +123,9 @@ class ContainerSerializer(LDListMixin, ListSerializer):
 
 
 class ManyJsonLdRelatedField(LDListMixin, ManyRelatedField):
-    pass
+    child_attr = 'child_relation'
+    url_field_name = "@id"
+
 
 
 class JsonLdField(HyperlinkedRelatedField):
@@ -182,6 +188,12 @@ class JsonLdIdentityField(JsonLdField):
     def get_value(self, dictionary):
         return super().get_value(dictionary)
 
+    def to_representation(self, value: Any) -> Any:
+        try:
+            return Hyperlink(value.webid(), value)
+        except AttributeError:
+            return super().to_representation(value)
+
 
 class LDPSerializer(HyperlinkedModelSerializer):
     url_field_name = "@id"
@@ -244,7 +256,7 @@ class LDPSerializer(HyperlinkedModelSerializer):
                         return super().get_value(obj)
                     else:
                         resource_id = Model.resource_id(self.parent.instance)
-                        obj = next(filter(lambda o: resource_id in o[self.parent.url_field_name], object_list))
+                        obj = next(filter(lambda o: resource_id.lstrip('/') in o[self.parent.url_field_name], object_list))
                         return super().get_value(obj)
                 except KeyError:
                     return super().get_value(dictionary)
@@ -358,7 +370,7 @@ class LDPSerializer(HyperlinkedModelSerializer):
             if item is empty:
                 return empty
             try:
-                full_item = next(filter(lambda o: item[self.url_field_name] == o[self.url_field_name], object_list))
+                full_item = next(filter(lambda o: self.url_field_name in o and (item[self.url_field_name] == o[self.url_field_name]), object_list))
             except StopIteration:
                 pass
             if full_item is None:
@@ -390,9 +402,22 @@ class LDPSerializer(HyperlinkedModelSerializer):
                     getattr(instance, field_name).add(related)
 
     def internal_create(self, validated_data, model):
+
+        nested_fk_fields_name = list(filter(lambda key: isinstance(validated_data[key], dict), validated_data))
+        for field_name in nested_fk_fields_name:
+            field_dict = validated_data[field_name]
+            field_model = getattr(model, field_name).field.rel.model
+            slug_field = Model.slug_field(field_model)
+            if slug_field in field_dict:
+                kwargs = {slug_field: field_dict[slug_field]}
+                sub_inst = field_model.objects.get(**kwargs)
+            else:
+                sub_inst = self.internal_create(field_dict, field_model )
+            validated_data[field_name] = sub_inst
+
         nested_fields = []
-        nested_fields_name = list(filter(lambda key: isinstance(validated_data[key], list), validated_data))
-        for field_name in nested_fields_name:
+        nested_list_fields_name = list(filter(lambda key: isinstance(validated_data[key], list), validated_data))
+        for field_name in nested_list_fields_name:
             nested_fields.append((field_name, validated_data.pop(field_name)))
 
         instance = model.objects.create(**validated_data)
@@ -429,8 +454,11 @@ class LDPSerializer(HyperlinkedModelSerializer):
         for (field_name, data) in nested_fields:
             manager = getattr(instance, field_name)
             slug_field = Model.slug_field(instance)
+            try:
+                item_pk_to_keep = list(map(lambda e: e[slug_field], filter(lambda x: slug_field in x, data)))
+            except TypeError:
+                item_pk_to_keep = list(map(lambda e: getattr(e, slug_field), filter(lambda x: hasattr(x, slug_field), data)))
 
-            item_pk_to_keep = list(map(lambda e: e[slug_field], filter(lambda x: slug_field in x, data)))
             for item in list(manager.all()):
                 if not str(getattr(item, slug_field)) in item_pk_to_keep:
                     if getattr(manager, 'through', None) is None:
@@ -439,10 +467,13 @@ class LDPSerializer(HyperlinkedModelSerializer):
                         manager.remove(item)
 
             for item in data:
-                if slug_field in item:
+                if not isinstance(item, dict):
+                    item.save()
+                    saved_item = item
+                elif slug_field in item:
                     kwargs = {slug_field: item[slug_field]}
-                    oldObj = manager.model.objects.get(**kwargs)
-                    savedItem = self.update(instance=oldObj, validated_data=item)
+                    old_obj = manager.model.objects.get(**kwargs)
+                    saved_item = self.update(instance=old_obj, validated_data=item)
                 else:
                     rel = getattr(instance._meta.model, field_name).rel
                     try:
@@ -451,7 +482,7 @@ class LDPSerializer(HyperlinkedModelSerializer):
                             item[reverse_id] = instance.pk
                     except AttributeError:
                         pass
-                    savedItem = self.internal_create(validated_data=item, model=manager.model)
+                    saved_item = self.internal_create(validated_data=item, model=manager.model)
 
                 if getattr(manager, 'through', None) is not None and manager.through._meta.auto_created:
-                    manager.add(savedItem)
+                    manager.add(saved_item)
